Package: labstatR
Version: 1.0.14
Title: Libreria Del Laboratorio Di Statistica Con R
Authors@R: c(person(given = "Stefano",
                    family = "M. Iacus",
                    role = c("aut", "cre"),
                    email = "siacus@iq.harvard.edu"),
             person(given = "Guido",
                    family = "Masarotto",
                    role = "aut",
                    email = "guido@sirio.stat.unipd.it"))
Description: Insieme di funzioni di supporto al volume "Laboratorio di
        Statistica con R", Iacus-Masarotto, MacGraw-Hill Italia, 2006.
        This package contains sets of functions defined in "Laboratorio
        di Statistica con R", Iacus-Masarotto, MacGraw-Hill Italia,
        2006. Function names and docs are in italian as well.
License: GPL (>= 2)
Depends: R (>= 2.10)
NeedsCompilation: no
Packaged: 2026-05-29 13:30:58 UTC; jago
Author: Stefano M. Iacus [aut, cre],
  Guido Masarotto [aut]
Maintainer: Stefano M. Iacus <siacus@iq.harvard.edu>
Repository: CRAN
Date/Publication: 2026-05-29 17:00:14 UTC
Built: R 4.6.0; ; 2026-05-29 17:28:50 UTC; unix
