# Default NAMESPACE created by R
# Remove the previous line if you edit this file

# Export all names
exportPattern(".")
export("kurt")
importFrom(stats, cov, runif, median, var, qchisq, optim, pchisq, rnorm)
importFrom(grDevices, rainbow)
importFrom(graphics, plot, axis, box, points, lines, abline, rect,polygon, hist, curve, symbols)
